class JWP_Ball extends KActor notplaceable;

/* References to the two AudioComponents used by a ball. */
var AudioComponent CollisionSound;
var AudioComponent RollSound;

/* The range of volume the rolling sound can reach. */
var float RollVolumeMinimum;
var float RollVolumeMaximum;

/* The pitch range. */
var float RollPitchMinimum;
var float RollPitchMaximum;

/* The range of velocity the ball will normalise for use by the volume and pitch. */
var float RollMinimum;
var float RollMaximum;

/* Thresholds for enabling/disabling the three emitters a ball has. */
var float FireThreshold;
var float SmokeThreshold;
var float SparksThreshold;

/* Keep references to the emitters so we can manipulate them. */
var ParticleSystemComponent FireEmitter;
var ParticleSystemComponent SmokeEmitter;
var ParticleSystemComponent SparksEmitter;

/* We can't tell it to activate/deactivate repeatedly because it'll get confused. */
var bool FirePlaying;
var bool SmokePlaying;
var bool SparksPlaying;
var bool AntiJam;

/* bFinished doesn't trigger until the sound has been played at least once    */
/* so we need to keep track of whether we've played the sound yet.            */
var bool bSoundPlayed;

/* The velocity of the ball when frozen. */
var vector PauseVelocity;
var bool Update;

/* Ran by the gameinfo when the object should be frozen.                      */
/*----------------------------------------------------------------------------*/
function Pause()
{

    Update = false;

    SetPhysics( PHYS_None );

}

/* Ran by the gameinfo when the object should be unfrozen.                    */
/*----------------------------------------------------------------------------*/
function Unpause()
{

    SetPhysics( PHYS_RigidBody );

    StaticMeshComponent.SetRBLinearVelocity( PauseVelocity );

    Update = true;

}

/* Executed whenever the KActor has a collision with a force > 128.0f.        */
/*----------------------------------------------------------------------------*/
event RigidBodyCollision( PrimitiveComponent HitComponent, PrimitiveComponent OtherComponent, const out CollisionImpactData RigidCollisionData, int ContactIndex )
{

    /* Normalise the collision strength from 128.0f ... 10128.0f to 0.0f to 1.0f. */
    local float Volume;

    Volume = VSize( RigidCollisionData.TotalNormalForceVector ) - 128.0f;

    Volume /= 10000.0f;

    if( Volume > 1.0f ) Volume = 1.0f;

    /* If the sound has finished or hasn't been played yet, set the volume and play. */
    if( CollisionSound.bFinished || ( !bSoundPlayed ) )
    {

        CollisionSound.VolumeMultiplier = Volume;
        CollisionSound.Stop();
        CollisionSound.Play();

        bSoundPlayed = true;

    }

}

/* Executed at gameplay start.                                                */
/*----------------------------------------------------------------------------*/
simulated event PostBeginPlay()
{

    /* Do what the superclass wants to in this instance. */
    Super.PostBeginPlay();

    /* Play the rolling sound now.  If we didn't do this manually it'd play   */
    /* while in editor without the volume adjustment in Tick().               */
    RollSound.Play();

}

/* Executed when the ball is destroyed - such as falling through KillZ. */
simulated event Destroyed()
{

    /* Notify the gametype so it can act. */
    JWP_Gameinfo( WorldInfo.Game ).BallDestroyed();

}

`define UpdateEmitter( name ) \
\
    if( Speed > `name Threshold )\
    {\
    \
        if( !`name Playing ) `name Emitter.ActivateSystem();\
        \
        `name Playing = true;\
    \
    }\
    else\
    {\
    \
        if( `name Playing ) `name Emitter.DeactivateSystem();\
        \
        `name Playing = false;\
    \
    }

/* Executed every frame.                                                      */
/*----------------------------------------------------------------------------*/
event Tick( float DeltaTime )
{

    local float Speed;

    /* If our superclass has something to do, do it. */
    Super.Tick( DeltaTime );

    /* Normalise the velocity to 0.0f ... 1.0f. */
    Speed = Vsize( Velocity );

    /* Switch our emitters on and off. */
    if( AntiJam )
    {

        `UpdateEmitter( Fire )
        `UpdateEmitter( Smoke )
        `UpdateEmitter( Sparks )

        AntiJam = false;

    }
    else AntiJam = true;

    Speed = ( Speed - RollMinimum ) / ( RollMaximum - RollMinimum );

    if( Speed < 0.0f ) Speed = 0.0f;
    if( Speed > 1.0f ) Speed = 1.0f;

    /* Apply the normalised levels to the AudioComponent. */
    RollSound.PitchMultiplier = lerp( RollPitchMinimum, RollPitchMaximum, Speed );
    RollSound.VolumeMultiplier = lerp( RollVolumeMinimum, RollVolumeMaximum, Speed );

    /* In case we pause. */
    if( Update )
    {

        PauseVelocity = Velocity;

    }

}

defaultproperties
{

    /* Configure the StaticMeshComponent that already exists to generate          */
    /* RigidBodyCollision events and give it our ball mesh.                       */
    /*----------------------------------------------------------------------------*/
    Begin Object Name=StaticMeshComponent0

        StaticMesh = StaticMesh'JWPContent.Ball';
        bNotifyRigidBodyCollision = true;
        ScriptRigidBodyCollisionThreshold = 128.0f;

        Scale = 0.7f;

    End Object

    /* Create an AudioComponent that loops for the rolling sound.                 */
    /*----------------------------------------------------------------------------*/
    Begin Object Class=AudioComponent Name=AudioComponent0

        /* Though it may seem like a good idea to play a looping sound right away, this means it plays in the editor even with game mode off. */
        bAutoPlay = false;
        bStopWhenOwnerDestroyed = true;
        bShouldRemainActiveIfDropped = true;
        VolumeMultiplier = 0.0f;

        SoundCue = SoundCue'JWPContent.BallRoll';

    End Object

    /* Keep a reference to it so we can manipulate it later. */
    RollSound = AudioComponent0;

    Components.Add( AudioComponent0 );
    /*----------------------------------------------------------------------------*/

    /* Create another AudioComponent for collisions.                              */
    /*----------------------------------------------------------------------------*/
    Begin Object Class=AudioComponent Name=AudioComponent1

        bAutoPlay = false;
        bStopWhenOwnerDestroyed = true;
        bShouldRemainActiveIfDropped = true;
        VolumeMultiplier = 0.0f;

        SoundCue = SoundCue'JWPContent.Bang';

    End Object

    /* Keep a reference so it can be player later. */
    CollisionSound = AudioComponent1;

    Components.Add( AudioComponent1 );
    /*----------------------------------------------------------------------------*/

    /* A ParticleSystemComponent used to make a fire trail.                            */
    /*----------------------------------------------------------------------------*/
    Begin Object Class=ParticleSystemComponent Name=ParticleSystemComponent0

        Template = ParticleSystem'JWPContent.FireRibbon'

    End Object;

    FireEmitter = ParticleSystemComponent0;

    Components.Add( ParticleSystemComponent0 );
    /*----------------------------------------------------------------------------*/

    /* A ParticleSystemComponent used to make a smoke trail.                            */
    /*----------------------------------------------------------------------------*/
    Begin Object Class=ParticleSystemComponent Name=ParticleSystemComponent1

        Template = ParticleSystem'JWPContent.BallSmoke';

    End Object;

    SmokeEmitter = ParticleSystemComponent1;

    Components.Add( ParticleSystemComponent1 );
    /*----------------------------------------------------------------------------*/

    /* A ParticleSystemComponent used to make a spark trail.                            */
    /*----------------------------------------------------------------------------*/
    Begin Object Class=ParticleSystemComponent Name=ParticleSystemComponent2

        Template = ParticleSystem'JWPContent.BallSparks';

    End Object;

    SparksEmitter = ParticleSystemComponent2;

    Components.Add( ParticleSystemComponent2 );
    /*----------------------------------------------------------------------------*/

    /* Set the variables for the rolling sound. */
    RollVolumeMinimum = 0.0f;
    RollVolumeMaximum = 1.0f;
    RollPitchMinimum = 0.3f
    RollPitchMaximum = 2.5f;
    RollMinimum = 4.0f;
    RollMaximum = 6144.0f;

    bSoundPlayed = false;

    /* Set the thresholds for the effects. */
    FireThreshold = 1536.0f;
    SmokeThreshold = 512.0f;
    SparksThreshold = 1024.0f;

    FirePlaying = true;
    SmokePlaying = true;
    SparksPlaying = true;

    AntiJam = false;

    /* Seems necessary to trigger some collision events. */
    bCollideActors = true;

    /* Need to be set for the plunger to be capable of spawning a ball. */
    bNoDelete = false;
    bStatic = false;

    /* Otherwise it will sit where spawned! */
    bWakeOnLevelStart = true;

    /* Our pause antifreeze! */
    Update = true;
    PauseVelocity = ( X = 0.0f, Y = 0.0f, Z = 0.0f );

}
