class JWP_TableObject extends DynamicSMActor;

/* Custom 1D spring physics variables.                                           */
/*----------------------------------------------------------------------------*/
/* A decay value which is powered by the frame delta to produce a framerate independent low pass filter of the speed. */
var float PhysicsDamping;

/* The open and closed positions of the 1D spring.  Copied to the Target value by the subclasses when the TableObject wants to move. */
var float PhysicsOpen;
var float PhysicsClosed;

/* The position the spring is trying to move towards. */
var float PhysicsTarget;

/* The current position of the spring. */
var float PhysicsCurrent;

/* Hard limiters for the top and bottom of the spring to prevent bumpers sailing into the sky or sinking through the floor and such. */
var float PhysicsMinimum;
var float PhysicsMaximum;

/* How much velocity should be retained after a hard limiter collision? */
var float PhysicsLimitbounce;

/* A multiplier for the pull towards the target position, useful for making the response more violent. */
var float PhysicsSpeed;

/* The current movement speed of the spring. */
var float PhysicsVelocity;

/* Disables the physics update. */
var bool Paused;
/*----------------------------------------------------------------------------*/

/* Variables controlling the collision sounds for the TableObject.            */
/*----------------------------------------------------------------------------*/
/* bFinished only flags after the sound has been played at least once so this is a bypass for it that only works once. */
var bool CollisionPlayed;

/* The upper and lower bounds for the collision strength, with the minimum being 0.0f volume and the maximum being 1.0f volume. */
var float CollisionMinimum;
var float CollisionMaximum;

/* Retains a reference to the AudioComponent used to play a collision sound. */
var AudioComponent CollisionSound;
/*----------------------------------------------------------------------------*/

/* Ticks the custom 1D spring physics.  Usually called from the Tick() event  */
/* by child classes' Super.Tick() reference but if for some reason it isn't   */
/* possible (if the child wants to bypass behaviour of its superclass for     */
/* instance or it needs to be manually updated outside of Tick() it can be.   */
/*----------------------------------------------------------------------------*/
function PhysicsUpdate( float DeltaSeconds )
{

    local float Computed;

    Computed = PhysicsDamping ** DeltaSeconds;

    PhysicsVelocity = lerp( ( PhysicsTarget - PhysicsCurrent ) * PhysicsSpeed, PhysicsVelocity, Computed );

    PhysicsCurrent += PhysicsVelocity * DeltaSeconds;

    if( PhysicsCurrent > PhysicsMaximum ){ PhysicsCurrent = PhysicsMaximum; PhysicsVelocity = -abs( PhysicsVelocity ) * PhysicsLimitbounce; }
    if( PhysicsCurrent < PhysicsMinimum ){ PhysicsCurrent = PhysicsMinimum; PhysicsVelocity = abs( PhysicsVelocity ) * PhysicsLimitbounce; }

}

/* Executed once per frame.                                                   */
/*----------------------------------------------------------------------------*/
simulated event Tick( float DeltaSeconds )
{

    /* Do anything the superclass wanted to in this event. */
    Super.Tick( DeltaSeconds );

    /* Tick the physics. */
    if( !Paused ) PhysicsUpdate( DeltaSeconds );

}

/* In the event of a collision over our theshold, this event is executed.     */
/*----------------------------------------------------------------------------*/
event RigidBodyCollision( PrimitiveComponent HitComponent, PrimitiveComponent OtherComponent, const out CollisionImpactData RigidCollisionData, int ContactIndex )
{

    local float Volume;

    /* Normalise the collision strength to 0.0f ... 1.0f using CollisionMinimum and CollisionMaximum as limits. */
    Volume = ( VSize( RigidCollisionData.TotalNormalForceVector ) - CollisionMinimum ) / ( CollisionMaximum - CollisionMinimum );

    if( Volume > 1.0f ) Volume = 1.0f;

    /* Only play the sound if it's finished or we haven't played it yet. */
    if( CollisionSound.bFinished || ( !CollisionPlayed ) )
    {

        CollisionSound.VolumeMultiplier = Volume;
        CollisionSound.Stop();
        CollisionSound.Play();

	    CollisionPlayed = true;

    }

}

defaultproperties
{

    /* Setup the collision of the inherited StaticMeshComponent to interact with the Ball Kactor and trigger RigidBodyCollision events. */
    /*----------------------------------------------------------------------------*/
    Begin Object Name=StaticMeshComponent0

        BlockRigidBody = true;
        RBChannel = RBCC_GameplayPhysics;
        RBCollideWithChannels = ( Default = TRUE, BlockingVolume = TRUE, GameplayPhysics = TRUE, EffectPhysics = TRUE );
        bNotifyRigidBodyCollision = true;
        ScriptRigidBodyCollisionThreshold = 128.0f;

    End Object
    /*----------------------------------------------------------------------------*/

    /* Some "sensible" defaults for the 1D spring physics.  Subclasses will likely change these to suit their needs. */
    /*----------------------------------------------------------------------------*/
    PhysicsDamping = 0.0001f;
    PhysicsOpen = 1.0f;
    PhysicsClosed = 0.0f;
    PhysicsCurrent = 0.0f;
    PhysicsTarget = 0.0f;
    PhysicsMinimum = -0.1f;
    PhysicsMaximum = 1.1f;
    PhysicsLimitbounce = 0.3f;
    PhysicsVelocity = 0.0f;
    PhysicsSpeed = 100.0f;

    Paused = false;
    /*----------------------------------------------------------------------------*/

    /* Create an AudioComponent for playing collision sound effects.              */
    /*----------------------------------------------------------------------------*/
    Begin Object Class=AudioComponent Name=AudioComponent0

        bAutoPlay = false;
        bStopWhenOwnerDestroyed = true;
        bShouldRemainActiveIfDropped = true;
        VolumeMultiplier = 0.0f;

    End Object

    CollisionSound = AudioComponent0;

    Components.Add( AudioComponent0 );

    CollisionPlayed = false;
    CollisionMinimum = 128.0f;
    CollisionMaximum = 10000.0f;
    /*----------------------------------------------------------------------------*/

    /* This is a dynamic object, but we're controlling the physics, not PhysX.    */
    /*----------------------------------------------------------------------------*/
    Physics = PHYS_Interpolating;
    bStatic = false;
    bBlockActors = true;
    /*----------------------------------------------------------------------------*/

}
